@@ -122,11 +122,6 @@ sub add_handle($$)
 	my $multi = shift;
 	my $easy = shift;
 
-	# Irssi won't allow timeout smaller than 10ms
-	Irssi::timeout_add_once( 10, sub {
-		$multi->socket_action();
-	}, '' );
-
 	$multi->{active} = -1;
 	$multi->SUPER::add_handle( $easy );
 }
@@ -151,9 +146,9 @@ sub socket_action
 	}
 }
 
-sub loop
+sub get_one
 {
-	warn __PACKAGE__ . " does not support blocking loop()\n";
+	warn __PACKAGE__ . " does not support blocking\n";
 	return;
 }
 
